@@ -13,37 +13,10 @@
 
 
 import os
-import time
-
-EXTENSIONS = []
-
-
-# - Functions ----------------------------------------
-
-
-def fix_path():
-    # To fix open shell (call run() without sys.argv) or self.load_extension(name) will not work.
-    import sys
-    path = os.path.dirname(__file__)
-    if path not in sys.path:
-        sys.path.insert(0, path)
-
-
-def wrap_function(func, before=(), after=()):
-    def wrapper(*args, **kwargs):
-        # print(func.__name__) # for test
-        [f() for f in before]
-        ret = func(*args, **kwargs)
-        [f() for f in after]
-        return ret
-    return wrapper
-
-
-# - Calltip ----------------------------------------
-
 
 import idlelib.calltip
 
+EXTENSIONS = []
 
 # idlelib.calltip._MAX_LINES = 999
 # idlelib.calltip._MAX_COLS = 999
@@ -63,8 +36,6 @@ class Calltip(idlelib.calltip.Calltip):
 idlelib.calltip.Calltip = Calltip
 
 
-# - AutoComplete ----------------------------------------
-
 
 import keyword
 import idlelib.autocomplete
@@ -83,36 +54,27 @@ class MyAutoComplete(idlelib.autocomplete.AutoComplete):
 idlelib.autocomplete.AutoComplete = MyAutoComplete
 
 
-# - IOBinding ----------------------------------------
 
 
-import idlelib.iomenu
-from idlelib.iomenu import IOBinding
+def FixPath():
+    # To fix open shell (call run() without sys.argv) or self.load_extension(name) will not work.
+    import sys
+    path = os.path.dirname(__file__)
+    if path not in sys.path:
+        sys.path.insert(0, path)
 
 
-class MyIOBinding(IOBinding):
-    def __init__(self, editwin):
-        # F5保存时，调用idlelib.runscript.getfilename()，设置自动保存时进入self.editwin.io.save(None)进行保存
-        self.save = wrap_function(self.save, after=editwin.after_save)
-        IOBinding.__init__(self, editwin)
-
-    def defaultfilename(self, mode="open"):
-        if not self.filename:
-            return self.dirname, time.strftime('Untitled_%Y%m%d_%H%M%S.py')
-        return super().defaultfilename(mode)
-
-
-idlelib.iomenu.IOBinding = MyIOBinding
-
-
-# - EditorWindow ----------------------------------------
-
+import idlelib.editor
+from idlelib.editor import EditorWindow
+from idlelib.mainmenu import menudefs
 
 # editor.EditorWindow -> pyshell.PyShellEditorWindow
 # editor.EditorWindow -> outwin.OutputWindow -> pyshell.PyShell
 
-import idlelib.editor
-from idlelib.editor import EditorWindow
+
+mymenudef = ('advance', [])
+
+menudefs.append(('advance', []))
 
 
 class MyEditorWindow(EditorWindow):
@@ -120,30 +82,46 @@ class MyEditorWindow(EditorWindow):
         if ('advance', 'Advance') not in self.menu_specs:
             self.menu_specs.append(('advance', 'Advance'))
 
-        self.before_copy = []
-        self.after_save  = []
-        self.after_close = []
-
-        # must before text binding, so before `EditorWindow.__init__()`
-        self.cut   = wrap_function(self.cut, before=self.before_copy)  # same as `copy`
-        self.copy  = wrap_function(self.copy, before=self.before_copy)
-        self.close = wrap_function(self.close, before=self.after_close)  # "<<close-window>>"事件不命中点击窗口关闭事件
-
         EditorWindow.__init__(self, *args)
 
         self.amenu = self.menudict['advance']
         self.make_rmenu() # make "self.rmenu"
 
-        self.recent_files_menu['postcommand'] = self.update_recent_files_list  # fix list not refresh when open another IDLE.
+        self.before_copy = []
+        self.after_save  = []
+        self.after_close = []
+
+        text = self.text
+        text.bind('<<save-window>>', self.save) # fix all event handle in this class.
+        text.bind('<F12>', self.Test)
+
+        self.recent_files_menu['postcommand'] = self.update_recent_files_list # fix list not refresh when open another IDLE.
 
         self.load_adv_extensions()
-        self.text.bind('<F12>', self.test)
+
+    def cut(self, event):
+        [f() for f in self.before_copy] # same as `copy`
+        return super().cut(event)
+
+    def copy(self, event):
+        [f() for f in self.before_copy]
+        return super().copy(event)
 
     def _close(self):
         print('handle with edit _close:', self.io.filename)
         super()._close()
         # raise # TODO 是否还有别的方法阻止清空剪切
 
+    def close(self):
+        # "<<close-window>>"事件不命中点击窗口关闭事件
+        print('handle with edit close:', self.io.filename)
+        [f() for f in self.after_close]
+        super().close()
+
+    def save(self, e):
+        self.io.save(e)
+        [f() for f in self.after_save]
+
     def add_adv_menu(self, label, sub, index='end', sp=False):
         menu = self.menudict['advance']
         if sp and menu.index('end') is not None:
@@ -168,7 +146,7 @@ class MyEditorWindow(EditorWindow):
         if menu.type('end') == 'separator':
             menu.delete('end')
 
-    def test(self, e):
+    def Test(self, e):
         print('editor on test')
         print('mark_names:', self.text.mark_names())
         print('tag_names:', self.text.tag_names())
@@ -176,16 +154,11 @@ class MyEditorWindow(EditorWindow):
 
 
 idlelib.editor.EditorWindow = MyEditorWindow
-
-
-# - Main ----------------------------------------
-
-
 from idlelib.pyshell import main  # must after hot patch
 
 
 def run(filename=None, exts=()):
-    fix_path()
+    FixPath()
     EXTENSIONS.extend(exts)
     if not EXTENSIONS:
         EXTENSIONS.extend(file for file in os.listdir(os.path.dirname(__file__)))
