@@ -7,6 +7,6 @@ import imp
 os.chdir('..')
 for file in glob.iglob('*.py'):
     pkg = imp.load_package(file, file) # every package's `name` should not repeat.
-    print('- __%s__  '%file.replace('_', '\\_'))
-    print(str(pkg.__doc__).strip().replace('\n', '  \n'))
+    print('- ' + file)
+    print(str(pkg.__doc__).strip())
     print()
